<!-- #include virtual="/quickstart/winforms/include/header.inc" -->

<h4>Creating Controls</h4>
      
<P>This topic demonstrates how you can create Win Forms controls.  It 
covers the basics of creating a control.  In addition, this topic will 
also cover adding painting logic to a control, exposing properties and 
events, using control licensing, and adding design-time behavior to your 
control.  The following topic areas will be covered in this topic:</P> 

<ul>
    <li><a target="content" href="#Introduction">Introduction</a>
    <li><a target="content" href="#Getting Started">Getting Started</a>
    <li><a target="content" href="#Adding Properties, Events and MetaData">Adding Properties, Events and MetaData</a>
    <li><a target="content" href="#Customising the property browser">Adding custom editors to the property browser</a>
    <li><a target="content" href="#Extender Providers">Extender Providers</a>
    <li><a target="content" href="#User Controls">User Controls</a>
    <li><a target="content" href="#Licensing Controls">Licensing Controls</a>
</ul>


<a name="Introduction">

<P><B>Note: </B>This topic is coverd in much more detial in the control 
creation section of the COM+ 2.0 SDK documentation.</P> 
      
<P>There are two types of controls: 

<UL> 
        
<LI><B>"Custom" Controls: </B>Controls that display UI by making calls to 
a <b> Graphics</b> object in the paint event.  Custom controls typically 
derive from <B>RichControl</B>.  A Chart control is an example of a custom 
control.  There is limited design time support for creating custom 
controls.  <BR><BR> 
        
<LI><B>"User" or Composite Controls: </B>Controls that are composed of 
other controls.  User Controls derive from <B>UserControl</B>.  A control 
that displays a customer address using TextBox controls is an example of a 
User Control.  There is full design time support for creating user 
controls using the WinDes application.  </LI>
</UL> 
      
<P>The majority of the samples in this topic demonstrate building custom 
controls,&nbsp; however the sections on exposing properties and events, 
defining design-time behavior, and licensing apply to both custom and User 
Controls.  </P> 
      
<a name="Getting Started">
<h5>Getting Started</h5>

<h6>Writing a simple control</h6>

<P>The following example creates a simple control that displays the value 
of its <b> Text</b> property by handling the <b>Paint</b> event.  In order 
to create this control and handle the event,&nbsp; must create a class 
that inherits from <b> RichControl</b> and create a method overrides the 
<b> OnPaint</b> method:</P> 
      
<Code:CodeSnippet runat=server>
<template name="CsTemplate">
    <xmp class=code1>
    
    public class HelloWorldControl : RichControl {
    
       protected override void OnPaint(PaintEventArgs e) {

           //Paint the Text property on the control</span>
           e.Graphics.DrawString(Text, 
                                 Font, 
                                 new SolidBrush(ForeColor), 
                                 ClientRectangle);
       }
   }

   </xmp>
</template>

<template name="VBTemplate">
   <xmp class=code1>
    
    public class HelloWorldControl 
        Inherits RichControl 
        
        Overrides Protected Sub OnPaint(e As PaintEventArgs) 

            'Paint the Text property on the control
            Dim rect As RectangleF = new RectangleF(ClientRectangle.X, ClientRectangle.Y,ClientRectangle.Width,ClientRectangle.Height) 
            e.Graphics.DrawString(Me.Text, Font, new SolidBrush(ForeColor), rect)
            
       End Sub
       
   End Class
   
   </xmp>
</template>

</Code:CodeSnippet>

<p>To view and run this sample:
<br><br>
<Code:CodeSnippet runat=server>
    
    <template name="CsTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/winforms/Samples/Cs/WritingControls/HelloWorldControl" 
        ViewSource="/quickstart/winforms/Samples/Cs/WritingControls/HelloWorldControl/HelloWorldControl.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="C# Hello World Control"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
    <template name="VBTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/winforms/Samples/VB/WritingControls/HelloWorldControl" 
        ViewSource="/quickstart/winforms/Samples/VB/WritingControls/HelloWorldControl/HelloWorldControl.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="VB Hello World Control"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
</Code:CodeSnippet>

<h6>Examining the design-time behavior of a control</h6>

<P>You can test the design time behavior of a control in WinDes:</P>
  <ol>
  <LI>Start the WinDes application.
  <LI>Add a new form by clicking <b>New VB</b> or <b>New C#</b> from the <b>File</b>
    menu.&nbsp; 
  <LI>Click <B>Add Library</B> on the Edit menu. 
  <LI>Select the DLL containing the control you want to use.<br>
    The control will appear at the bottom of the Toolbox. 
  <LI>Select the control and add it to the form.<br>
    You will see the control appear on the form 
  <LI>If you are adding the control from the previous example you will
    notice that even this very simple control has a full set of 
  properties and a broad range of design time behavior. This default 
  behavior is inherited from the <b>RichControl </b>class. </LI>
  </ol>
  <P><B>Note: </B>For a control to be displayed in the designer it must have a public 
constructor that takes no parameters.</P>
      
<a name="Adding Properties, Events and MetaData">
<h5>Adding Properties, Events and MetaData</h5>

      <P>Now that we have a simple control created, we can add properties,
      events, and metadata information to it. The following example: 
      <UL>
        <LI>Adds a property called <B>DrawMode</B> that is used to determine how 
        the control paints. 
        <LI>Adds an event that fires when the <B>DrawMode</B> property is 
        changed. 
        <LI>Adds "metadata" to the control to describe its design-time behavior. 
        <LI>Overrides a property inherited from <b> RichControl</b> to hide it at 
        design-time. </LI></UL>

<h6>Adding a property</h6>

<P>First, we need to create a simple enumeration called <b>DrawingMode</b>:</P>

<Code:CodeSnippet runat=server>
<template name="CsTemplate">
    <xmp class=code1>
    
    public enum DrawingMode {
        Happy = 0,
        Sad = 1,
        Angry = 2
    }
    </xmp>
</template>
<template name="VBTemplate">
   <xmp class=code1>
   
    Public Enum DrawingModeStyle 
        Happy = 0
        Sad = 1
        Angry = 2
    End Enum

   </xmp>
</template>
</Code:CodeSnippet>

      <P>Next we need to add a <b> DrawingMode</b> property to the control. The
      following code adds this property to our simple control we created
      previously:</P>

<Code:CodeSnippet runat=server>
<template name="CsTemplate">
    <xmp class=code1>

    public DrawingMode DrawingMode {
        get { return drawingMode;}
        set { 
            drawingMode=value;

            //Set BackColor and ForeColor based on DrawingMode</span>
            SetColors();

            //Raise property changed event for DrawingMode</span>
            RaisePropertyChangedEvent("DrawingMode");
        }
    }
    </xmp>
</template>

<template name="VBTemplate">
   <xmp class=code1>
    'DrawingMode - controls how the control paints
    Public Property DrawingMode As DrawingModeStyle
        Get 
            return myDrawingMode
        End Get
        
        Set
            myDrawingMode=value

            'Set BackColor and ForeColor based on DrawingMode
            SetColors

            'Raise property changed event for DrawingMode
            RaisePropertyChangedEvent("DrawingMode")
        End Set 
        
    End Property
   
   </xmp>
</template>
</Code:CodeSnippet>

<P><B>Note</B> that the property <b> set</b> code includes a call to the 
<B>RaisePropertyChangedEvent</B> method.  This method raises a property 
change notification event.  It is important to raise the property change 
notification event since the Win Forms designer listens to this event.  It 
listens to this event so that it knows when a property has changed.  Later 
in the code our control will take advantage of this event.</P> 

<P>The call to the <B>SetColors</B> method simply sets the BackColor and 
ForeColor of the control based on the value of <B>DrawingMode</B>.  Add 
the following code to the control:</P> 
      
<Code:CodeSnippet runat=server>
<template name="CsTemplate">
    <xmp class=code1>

    private void SetColors() {

        switch (drawingMode) {

           case DrawingMode.Happy:
               base.BackColor = Color.Yellow ;
               base.ForeColor = Color.Green ;
               break ;

           case DrawingMode.Sad:
               base.BackColor = Color.LightSlateGray ;
               base.ForeColor = Color.White ;
               break ;

          case DrawingMode.Angry:
              base.BackColor = Color.Red ;
              base.ForeColor = Color.Teal ;
              break ;

           default:
               base.BackColor = Color.Black ;
               base.ForeColor = Color.White ;
       }

    }
    </xmp>
</template>

<template name="VBTemplate">
   <xmp class=code1>

        Private Sub SetColors() 
 
            Select Case myDrawingMode
 
               Case DrawingModeStyle.Happy
                   MyBase.BackColor = Color.Yellow 
                   MyBase.ForeColor = Color.Green 
                    
               Case DrawingModeStyle.Sad
                   MyBase.BackColor = Color.LightSlateGray 
                   MyBase.ForeColor = Color.White 
 
              Case DrawingModeStyle.Angry
                  MyBase.BackColor = Color.Red 
                  MyBase.ForeColor = Color.Teal 
 
              Case Else
                   MyBase.BackColor = Color.Black 
                   MyBase.ForeColor = Color.White 
                   
           End Select
 
        End Sub
   
   </xmp>
</template>
</Code:CodeSnippet>

<P>We can now use this information when the control paints its contents.  
Add this code to the control class to override the <b>OnPaint</b> method 
of <b>RichControl</b>:</P> 
      
<Code:CodeSnippet runat=server>
<template name="CsTemplate">
    <xmp class=code1>
    
    protected override void OnPaint(PaintEventArgs e) {

        e.Graphics.FillRectangle(new SolidBrush(BackColor), ClientRectangle);

        Size textSize = (Size) e.Graphics.MeasureString(Text, Font);

        float xPos = (ClientRectangle.Width/2) - (textSize.Width/2);
        float yPos = (ClientRectangle.Height/2) - (textSize.Height/2);

        e.Graphics.DrawString(Text, 
                              Font, 
                              new SolidBrush(ForeColor), 
                              xPos, yPos);

    }
    </xmp>
</template>

<template name="VBTemplate">
   <xmp class=code1>
   
    Overrides Protected Sub OnPaint(e As PaintEventArgs) 

        e.Graphics.FillRectangle(new SolidBrush(BackColor), ClientRectangle)

        Dim textSize As SizeF = e.Graphics.MeasureString(Me.Text, Font)

        Dim xPos As Single = CSng((ClientRectangle.Width/2) - (textSize.Width/2))
        Dim yPos As Single = CSng((ClientRectangle.Height/2) - (textSize.Height/2))

        e.Graphics.DrawString(Me.Text, Font,new SolidBrush(ForeColor),xPos, yPos)

    End Sub
   
   </xmp>
</template>
</Code:CodeSnippet>

<P><B>Note</B> in order to have the control automatically repaint when it 
is resized we need to set the <b> ResizeRedraw</b> style bit when the 
control is created.  Add the following constructor to our control's 
code:</P> 

<Code:CodeSnippet runat=server>
<template name="CsTemplate">
    <xmp class=code1>
    
    public SimpleControl() :base() {
        drawingMode = DrawingMode.Happy;
        SetColors();
        SetStyle(ControlStyles.ResizeRedraw, true);
    }
    </xmp>
</template>

<template name="VBTemplate">
   <xmp class=code1>
   
    Public Sub New() 
        MyBase.New
        myDrawingMode = DrawingModeStyle.Happy
        SetColors
        SetStyle(ControlStyles.ResizeRedraw, True)
    End Sub

   </xmp>
</template>
</Code:CodeSnippet>


<h6>Adding an Event</h6>

      <P>In this next step, we will add a simple event that is raised when the <b> DrawingMode</b>
      property changes. 
      There is already an event that is raised,&nbsp; the <b> PropertyChange</b> event, so why 
      do we add our own event? The <b> PropertyChange</b> event is 
      not exposed to control consumers by default,&nbsp; it is a hidden event used by 
      the framework infrastructure.</P>
      <P>Typically the first thing you do is to declare an <b> EventHandler</b> and
      <b> EventArgs</b> 
      class for your event. Because our event is simple we can use the standard 
      EventHandler and EventArgs classes.</P>
      <P>With our EventHandler and EventArgs classes declared we need to add
      methods that allow our event to be registered, unregistered, and raised. These follow a standard design pattern: 
      <UL>
        <LI><B>AddOn&lt;EventName&gt;</B> allows the user of the control to 
        register an event-handling method. 
        <LI><B>RemoveOn&lt;EventName&gt;</B> allows the user of the control to 
        remove an event-handling method. 
        <LI><B>On&lt;EventName&gt;</B> raises the event. Typically this method 
        is protected, only developers deriving from the control will need to 
        directly raise an event. </LI></UL>
      <p>Add the following methods and member variables to our control's source
      code:</p>

<Code:CodeSnippet runat=server>
<template name="CsTemplate">
    <xmp class=code1>
    
        public class SimpleControl : RichControl {

        private DrawingMode drawingMode ;
        private EventHandler drawingModeChanged; 

        ....

        //Add an event handler for the DrawingModeChanged event</span>
        public void AddOnDrawingModeChanged(EventHandler handler) {
            drawingModeChanged = (EventHandler)Delegate.Combine(drawingModeChanged, handler);
        }

        //Raise the DrawingModeChanged event</span>
        protected virtual void OnDrawingModeChanged(EventArgs e) {
            //DrawingMode has changed so redraw the control</span>
            //Done here so we have one centralised placed that causes a redraw</span>
            Invalidate();
            
            //If event handlers have been registered then notify them</span>
            if (drawingModeChanged != null) {
                drawingModeChanged(this, e);
            }
        }

        //Remove an event handler for the DrawingModeChanged event</span>
        public void RemoveOnDrawingModeChanged(EventHandler handler) {
            drawingModeChanged = (EventHandler)Delegate.Remove(drawingModeChanged, handler);
        }

        ....
    }

    </xmp>
</template>

<template name="VBTemplate">
   <xmp class=code1>

    Public Class SimpleControl 
        Inherits RichControl 

        Private myDrawingMode As DrawingModeStyle
        Private drawingModeChanged As EventHandler 
        ....

        Public Sub AddOnDrawingModeChanged(handler As EventHandler) 
            drawingModeChanged = CType(System.Delegate.Combine(drawingModeChanged, handler), EventHandler)
        End Sub
 
        Overridable Protected Sub OnDrawingModeChanged(e As EventArgs) 
            Invalidate
            If (drawingModeChanged <> Nothing) Then
                drawingModeChanged.Invoke(Me, e)
            End If
            
        End Sub
 
        Public Sub RemoveOnDrawingModeChanged(handler As EventHandler) 
            drawingModeChanged = CType(System.Delegate.Remove(drawingModeChanged, handler), EventHandler)
        End Sub
   
   </xmp>
</template>
</Code:CodeSnippet>

<P>Having added the event handler and event methods, we now need to raise 
the event.  We could go to the <b> DrawingMode</b> property <b> set</b> 
statement and add a call to <B>OnDrawingModeChanged</B>, however as this 
method already raises a property change notification event, we can simply 
listen for that event to fire.  Rather than adding an event-handling 
method we will override the <b> OnPropertyChanged</b> method.  Add the 
following code to our control:</P> 
      

<Code:CodeSnippet runat=server>
<template name="CsTemplate">
    <xmp class=code1>
    
        ....
        //Override OnPropertyChanged to raise the DrawingModeChanged event</span>
        protected override void OnPropertyChanged(PropertyChangedEventArgs e) {
            base.OnPropertyChanged(e);
            string d = e.PropertyName;

            if (d.Equals("DrawingMode")) {
                OnDrawingModeChanged(EventArgs.Empty);
            }
        }

        ....
    </xmp>
</template>

<template name="VBTemplate">
   <xmp class=code1>
   
    'Catch property changed event for DrawingMode to fire DrawingMoe changed 
    'and repaint the control
    Overrides Protected Sub OnPropertyChanged(e As PropertyChangedEventArgs) 
        MyBase.OnPropertyChanged(e)
        Dim d as string = e.PropertyName

        If (d.Equals("DrawingMode")) Then
            OnDrawingModeChanged(EventArgs.Empty)
        End If 
        
    End Sub 

   </xmp>
</template>
</Code:CodeSnippet>

<P>Everytime the property is changed, the control will re-draw and raise 
the <b> DrawingModeChanged</b> event.  

<h6>Using the control</h6>

<P>Now we have written our control, and added some behavior to it we can 
compile it into a DLL and use it in an application.  The&nbsp; new control 
can be used&nbsp; like any other Win Forms control.  The following example 
demonstrates how to use our control in an application:</P> 
      
<Code:CodeSnippet runat=server>
<template name="CsTemplate">
    <xmp class=code1>

    ....

    //Create the control and set its properties</span>
    simpleControl1 = new SimpleControl();
    simpleControl1.Size = new System.Drawing.Size(304, 328);
    simpleControl1.TabIndex = 0;
    simpleControl1.Anchor = System.WinForms.AnchorStyles.All;
    simpleControl1.Text = "Win Forms Mood Control";

    //Add an event handling method for the DrawingModeChanged event</span>
    simpleControl1.AddOnDrawingModeChanged(new System.EventHandler(simpleControl1_DrawingModeChanged));

    ....

    private void simpleControl1_DrawingModeChanged(object sender, System.EventArgs e) {
        MessageBox.Show("DrawingMode changed");
    }

    ....

    </xmp>
</template>

<template name="VBTemplate">
   <xmp class=code1>
    
    ....
    simpleControl1.Size = new System.Drawing.Size(304, 328)
    simpleControl1.TabIndex = 0
    simpleControl1.Anchor = System.WinForms.AnchorStyles.All
    simpleControl1.Text = "Win Forms Mood Control"
    ....
    simpleControl1.AddOnDrawingModeChanged(AddressOf simpleControl1_DrawingModeChanged)
    ....
    Private Sub simpleControl1_DrawingModeChanged(sender As object, e As System.EventArgs)
        MessageBox.Show("DrawingMode changed")
    End Sub

   
   </xmp>
</template>
</Code:CodeSnippet>


<h6>Adding design-time information for the control</h6>

<P>Now that we have a working control we can augment it with design time 
information that improves the usability of the control in the WinDes Form 
Designer.  Design time information is recorded using NGWS "meta-data" 
on the control binary.  This metadata is defined using a series of 
attribute classes from the <B>System.ComponentModel</B> namespace.  For 
example, the default event for our control is the <b> 
DrawingModeChanged</b> event.  When the user double-clicks on the control 
we want the form designer to add an event-handling method for our default 
event.  You register the default event using the <B>DefaultEvent</B> class 
attribute.  Similarly you can define the default property on a control 
using the <B>DefaultProperty</B> class attribute.  This attribute 
determines which property is given focus in the property browser by 
default.  Class attributes are declared as part of the class declaration.  
The following example demonstrates class attributes:</P> 

<Code:CodeSnippet runat=server>
<template name="CsTemplate">
    <xmp class=code1>
    
    [
    DefaultProperty("DrawingMode"),
    DefaultEvent("DrawingModeChanged"),
    ]
    public class SimpleControl : RichControl {
        ....
    }
    
    </xmp>
</template>

<template name="VBTemplate">
   <xmp class=code1>
    Public Class _
        <DefaultProperty("DrawingMode"), DefaultEvent("DrawingModeChanged")> _
        SimpleControl 
        Inherits RichControl 
    ....
    End Class   
   </xmp>
</template>
</Code:CodeSnippet>

<P>The next step is to add more design time information for the <b> 
DrawingMode</b> property.  Add the following attributes to our 
<b>DrawingMode</b> method:</P> 
      
<Code:CodeSnippet runat=server>
<template name="CsTemplate">
    <xmp class=code1>
    
    [
        Category("Appearance"),
        Description("Controls how the control paints"),
        DefaultValue(DrawingMode.Happy),
        Bindable(true),
    ]
    public DrawingMode DrawingMode {
        get { return drawingMode;}
        set { 
            drawingMode=value;

            //Set BackColor and ForeColor based on DrawingMode</span>
            SetColors();

            //Raise property changed event for DrawingMode</span>
            RaisePropertyChangedEvent("DrawingMode");
        }
    }

    
    </xmp>
</template>

<template name="VBTemplate">
   <xmp class=code1>
        
    'DrawingMode - controls how the control paints
    Public Property _
    <Category("Appearance"), _
     Description("Controls how the control paints"), _
     DefaultValue(DrawingModeStyle.Happy), _
     Bindable(true)> _
    DrawingMode As DrawingModeStyle
        
        Get 
            return myDrawingMode
        End Get
        
        Set
            myDrawingMode=value

            'Set BackColor and ForeColor based on DrawingMode
            SetColors

            'Raise property changed event for DrawingMode
            RaisePropertyChangedEvent("DrawingMode")
            
        End Set 
        
    End Property

   
   </xmp>
</template>
</Code:CodeSnippet>

<p>The attributes that we assigned to the method are as follows:</p>
<UL>
  <LI>The <b> Category</b> attribute determines the category for the property.
    This is used when the property is displayed in Categorized view in a
    Properties window. 
  <LI>The Description attribute sets the short description that is
    displayed 
  at the bottom of the Properties window when the property is selected. 
  <LI>The DefaultValue attribute is used by the designer to determine 
  whether a property value needs to be persisted. 
  <LI>The Bindable attribute is used to determine whether this property is
    displayed in the default data binding view. </LI></UL>
<P>We now need to add design time information for the <b> DrawingModeChanged</b>
event. Add the following meta-data information to the <b>AddOnDrawingModeChanged</b>
method in our code:</P>
      
<Code:CodeSnippet runat=server>
<template name="CsTemplate">
    <xmp class=code1>
    
        [Description("Raised when the DrawingMode changes")]
        public void AddOnDrawingModeChanged(EventHandler handler) {
            drawingModeChanged = (EventHandler)Delegate.Combine(drawingModeChanged, handler);
        }
    </xmp>
</template>

<template name="VBTemplate">
   <xmp class=code1>
   
   'DrawingModeChanged Event
    Public Sub _
    <Description("Raised when the DrawingMode changes")> _
    AddOnDrawingModeChanged(handler As EventHandler) 
        drawingModeChanged = CType(System.Delegate.Combine(drawingModeChanged, handler), EventHandler)
    End Sub

   </xmp>
</template>
</Code:CodeSnippet>

<P>In the last step we need to hide two properties.&nbsp; Because the <b> 
DrawingMode</b> determines what background and foreground colors are used 
to draw the control we don't want the user to be able to set the <b> 
BackColor</b> and <b>ForeColor</b> properties.  We cannot remove the 
properties because they are declared in the <b> RichControl</b> class, 
however we can hide them so that they do not show up in the property 
browser using the <B>Browsable</B> attribute.  Add the following code to 
our control class to override the two properties:</P> 
      
<Code:CodeSnippet runat=server>
<template name="CsTemplate">
    <xmp class=code1>
    
    //Remove the BackColor property from the property browser</span>
    [Browsable(false)]
    public Color BackColor { 
        override get {return base.BackColor;}  
        override set {} 
    }

    //Remove the ForeColor property from the property browser</span>
    [Browsable(false)]
    public Color ForeColor  { 
        override get {return base.ForeColor;}  
        override set {} 
    }
    </xmp>
</template>

<template name="VBTemplate">
   <xmp class=code1>
    
    'Remove the BackColor property from the property browser
    Overrides Public Property <Browsable(false)> BackColor As Color 
        Get
            return MyBase.BackColor
        End Get
        
        Set
            'No Action
        End Set
        
    End Property 

    'Remove the ForeColor property from the property browser 
    Overrides Public Property <Browsable(false)> ForeColor As Color 
        Get
            return MyBase.ForeColor
        End Get
        
        Set
            'No Action
        End Set
        
    End Property 

   </xmp>
</template>
</Code:CodeSnippet>

<h6>Examining the design-time behavior of a control</h6>

<P>You can test the design-time behavior of this control using the WinDes
application, following the same procedure that was used for the
HelloWorldControl demonstrated previously in this topic. Some things that
should be noted:</P>
<UL>
  <LI>The Properties window automatically understands how to edit the <b> 
  DrawingMode</b> property. Iit understands how to use the enumeration we
    defined for our property. 
  <LI>The Properties window displays descriptions for the <b> DrawingMode</b> 
  property and the <b> DrawingModeChanged</b> event. 
  <LI>The <b>ForeColor</b> and <b> BackColor</b> properties are not
    displayed in the Properties window. 
  <LI>Double-clicking on the control creates an event-handling method for 
  the <b> DrawingModeChanged</b> event. 
  <LI>Changing the <b> DrawingMode</b> property causes the control to change 
  immediately. 
  <LI>Changing the <b> DrawingMode</b> property from &quot;Happy&quot; to
    &quot;Sad&quot; causes the 
  property to be highlighted to indicate that it has changed. Changing <b> 
  DrawingMode</b> back to &quot;Happy&quot; causes the highlight to be removed. The
    Properties window uses the default value (&quot;Happy&quot;) to determine whether the 
  property has actually changed in value. </LI></UL>
<P>To view and run this sample: <BR><BR>

<Code:CodeSnippet runat=server>
    
    <template name="CsTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/winforms/Samples/Cs/WritingControls/SimpleControl" 
        ViewSource="/quickstart/winforms/Samples/Cs/WritingControls/SimpleControl/SimpleControl.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="C# SimpleControl"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
    <template name="VBTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/winforms/Samples/VB/WritingControls/SimpleControl" 
        ViewSource="/quickstart/winforms/Samples/VB/WritingControls/SimpleControl/SimpleControl.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="VB SimpleControl"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
</Code:CodeSnippet>

<a name="Customising the property browser">
<h5>Adding custom editors to the property browser</h5>

<P>In the previous sample we added a property that uses an enumeration to 
our control.  When we looked at this property at design time, the 
<B>Properties window</B> recognized that the property used an enumeration 
and automatically used an enumeration property editor.  But what does the 
property browser do if it does not recognize the type of the property?  
The NGWS Component Model includes a mechanism that allows a control author 
to create an editor for a property and register that editor as the editor 
for that type.  Again this is driven by meta data and attributes.  The 
following sample explains how add a custom property editor to a 
control.</P> 
      
<P>In this sample, we have written a <B>FlashTrackBar</B> control that is 
like the Windows <b> TrackBar</b> control but "Flashier".  It uses a GDI+ 
LinearGradientBrush to paint its contents.  The position of the 
FlashTrackBar is determined by the <b> Value</b> property.  This property 
has a custom property editor.  The following code demonstrates the 
property declaration: </p>

<Code:CodeSnippet runat=server>

<template name="CsTemplate">
    <xmp class=code1>
    
        [
            Category("Flash"), 
            Editor(typeof(FlashTrackBarValueEditor), typeof(UITypeEditor)),
            DefaultValue(0)
        ]
        public int Value {
        ....
        }
    </xmp>
</template>

<template name="VBTemplate">
   <xmp class=code1>
   
        Public Property _
            <Category("Flash"), _
             Editor(typeof(FlashTrackBarValueEditor), typeof(UITypeEditor)), _
             DefaultValue(0)> _
        Value As Integer 
            .... 
        End Property
   
   </xmp>
</template>

</Code:CodeSnippet>

<P>The Editor attribute informs that Win Forms designer that the property 
browser should use the class <B>FlashTrackBarValueEditor</B> as the custom 
property editor for this property.</P> 
      
<P>The property editor is implemented using a FlashTrackBar.  The most 
interesting methods in the FlashTrackBarValueEditor are:</P> 

<Code:CodeSnippet runat=server>
<template name="CsTemplate">
    <xmp class=code1>
    
        public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context) {
            if (context != null && context.Instance != null) {
                return UITypeEditorEditStyle.DropDown;
            }
            return base.GetEditStyle(context);
        }
    </xmp>
</template>

<template name="VBTemplate">
   <xmp class=code1>
   
    Overrides OverLoads Public Function GetEditStyle(context As ITypeDescriptorContext) As UITypeEditorEditStyle
        if (Not(context Is Nothing) AND Not(context.Instance Is Nothing)) Then
            return UITypeEditorEditStyle.DropDown
        End If
        return MyBase.GetEditStyle(context)
    End Function
   
   </xmp>
</template>

</Code:CodeSnippet>

<P>This method informs the property browser that the editor uses a 
drop-down style UI.</P> 

<Code:CodeSnippet runat=server>
<template name="CsTemplate">
    <xmp class=code1>
    
        public override object EditValue(ITypeDescriptorContext context, IServiceObjectProvider provider, object value) {

            if (context != null 
                && context.Instance != null
                && provider != null) {

                edSvc = (IWinFormsEditorService)provider.GetServiceObject(typeof(IWinFormsEditorService));

                if (edSvc != null) {
                    FlashTrackBar trackBar = new FlashTrackBar();
                    trackBar.AddOnValueChanged(new EventHandler(this.ValueChanged));
                    SetEditorProps((FlashTrackBar)context.Instance, trackBar);
                    bool asInt = true;
                    if (value is int) {
                        trackBar.Value = (int)value;
                    }
                    else if (value is byte) {
                        asInt = false;
                        trackBar.Value = (byte)value;
                    }
                    edSvc.DropDownControl(trackBar);
                    if (asInt) {
                        value = trackBar.Value;
                    }
                    else {
                        value = (byte)trackBar.Value;
                    }
                }
            }

            return value;
        }
    </xmp>
</template>

<template name="VBTemplate">
   <xmp class=code1>
   
   Overrides OverLoads Public Function EditValue(context As ITypeDescriptorContext, _
            provider As IServiceObjectProvider, value As object) As Object

       if (Not(context Is Nothing) AND Not(context.Instance Is Nothing) AND Not(provider Is Nothing)) Then

           edSvc = CType(provider.GetServiceObject(GetType(IWinFormsEditorService)),IWinFormsEditorService)

           if Not (edSvc Is Nothing) Then
               
               Dim trackBar As FlashTrackBar = new FlashTrackBar()
               trackBar.AddOnValueChanged(AddressOf Me.ValueChanged)
               SetEditorProps(CType(context.Instance,FlashTrackBar), trackBar)
               
               Dim asInt As Boolean = true
               
               if (TypeOf value Is Integer) Then
                   trackBar.Value = CInt(value)
               else if (TypeOf value Is System.Byte) Then
                   asInt = false
                   trackBar.Value = CType(value, Byte)
               End If
               
               edSvc.DropDownControl(trackBar)
               
               if (asInt) Then
                   value = trackBar.Value
               else 
                   value = CType(trackBar.Value, Byte)
               End If
               
           End If
           
       End If

        return value
       
    End Function
   
   </xmp>
</template>

</Code:CodeSnippet>

<p><b>EditValue</b> implements the editor UI.

<P><B>Note</B> FlashTrackBar also demonstrates the use of the 
<B>ShouldPersist&lt;PropertyName&gt;</B> pattern.  The DefaultValue 
attribute can only be used for simple types.  For complex types such as 
Colors you use the <B>ShouldPersist&lt;PropertyName&gt;</B> pattern.  For 
example:</P> 
      
<Code:CodeSnippet runat=server>
<template name="CsTemplate">
    <xmp class=code1>
    
        [
            Category("Flash")
        ]
        public Color EndColor {
            get {
                ....
            }
            set {
                ....
            }
        }

        public bool ShouldPersistEndColor() {
            return !(endColor == Color.LimeGreen);
        }
    </xmp>
</template>

<template name="VBTemplate">
   <xmp class=code1>
   
        Public Property <Category("Flash")> EndColor As Color
            ....
        End Property
 
       Public Function ShouldPersistEndColor() As Boolean
           return Not (myEndColor.Equals(Color.LimeGreen))
       End Function
   
   </xmp>
</template>

</Code:CodeSnippet>

<P>In the code above, the <b>Color.LimeGreen</b> value is the default 
value.  The designer calls the ShouldPersist method in order to determine 
whether to save the value of the EndColor property.</P> 

<P>To view and run this sample: <BR><BR> 

<Code:CodeSnippet runat=server>
    
    <template name="CsTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/winforms/Samples/Cs/WritingControls/CustomUITypeEditor" 
        ViewSource="/quickstart/winforms/Samples/Cs/WritingControls/CustomUITypeEditor/CustomUITypeEditor.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="C# CustomUITypeEditor"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
    <template name="VBTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/winforms/Samples/VB/WritingControls/CustomUITypeEditor" 
        ViewSource="/quickstart/winforms/Samples/VB/WritingControls/CustomUITypeEditor/CustomUITypeEditor.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="VB CustomUITypeEditor"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
</Code:CodeSnippet>

<P>You should examine the behavior of this control in WinDes in order to 
fully understand how the custom property editor works.</P> 
      
<a name="Extender Providers">
<h5>Extender Providers</h5>

<P>An Extender Provider is a component that provides properties to other 
components.  For example the ToolTip control is implemented as an Extender 
Provider.  When you add a ToolTip control to a Form, all other controls on 
the form have a ToolTip property added to their properties list.</P> 
      
<P>The following sample demonstrates how to build an Extender Provider by 
creating the HelpLabelControl.  The following code shows the 
implementation of the CanExtend method and the HelpText property.  The 
CanExtend method is used by the designer to determine whether to extend 
this property to a given control.  The HelpLabelControl extends the 
HelpText property for use with the controls on a form.  The help text for 
a control is displayed in a panel when the control has focus.</P> 
      
<Code:CodeSnippet runat=server>

<template name="CsTemplate">
    <xmp class=code1>
    
    bool IExtenderProvider.CanExtend(object target) {
        if (target is Control &&
            !(target is HelpLabel)) {

            return true;
        }
        return false;
    }
        
    [
    DefaultValue(""),
    ExtenderProperty(typeof(Control))
    ]
    public string GetHelpText(Control control) {
        string text = (string)helpTexts[control];
        if (text == null) {
            text = string.Empty;
        }
        return text;
    }
            
    </xmp>
</template>

<template name="VBTemplate">
   <xmp class=code1>
   Visual Basic Code not available in the PDC release. Please refer to the C# code.
   </xmp>
</template>

</Code:CodeSnippet>

<br><br>
<Code:CodeSnippet runat=server>
    
    <template name="CsTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/winforms/Samples/Cs/WritingControls/HelpLabelControl" 
        ViewSource="/quickstart/winforms/Samples/Cs/WritingControls/HelpLabelControl/HelpLabelControl.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="C# HelpLabel"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
    <template name="VBTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/winforms/Samples/VB/WritingControls/HelpLabelControl" 
        ViewSource="/quickstart/winforms/Samples/VB/WritingControls/HelpLabelControl/HelpLabelControl.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="VB HelpLabel"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
</Code:CodeSnippet>

<a name="User Controls">
<h5>User Controls</h5>
<p>This sample demonstrates how to build a User Control.</p>
<br>
<Code:CodeSnippet runat=server>

<template name="CsTemplate">
    <xmp class=code1>
    
    CustomerControl1.Anchor=System.WinForms.AnchorStyles.All;
    CustomerControl1.AutoScrollMinSize = new System.Drawing.Size(0, 0);
    CustomerControl1.Size = new System.Drawing.Size(400, 310);
    CustomerControl1.TabIndex = 0;
    </xmp>
</template>

<template name="VBTemplate">
   <xmp class=code1>
    
    CustomerControl1.Anchor=System.WinForms.AnchorStyles.All
    CustomerControl1.AutoScrollMinSize = new System.Drawing.Size(0, 0)
    CustomerControl1.Size = new System.Drawing.Size(400, 310)
    CustomerControl1.TabIndex = 0
   </xmp>
</template>

</Code:CodeSnippet>

<br><br>
<Code:CodeSnippet runat=server>
    
    <template name="CsTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/winforms/Samples/Cs/WritingControls/UserControl" 
        ViewSource="/quickstart/winforms/Samples/Cs/WritingControls/UserControl/UserControl.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="C# HelpLabel"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
    <template name="VBTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/winforms/Samples/VB/WritingControls/UserControl" 
        ViewSource="/quickstart/winforms/Samples/VB/WritingControls/UserControl/UserControl.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="VB UserControl"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
</Code:CodeSnippet>

<a name="Licensing Controls">
<h5>Licensing Controls</h5>

<P>The following sample demonstrates how to build a control that uses the 
default licensing provider.  The LicenseProvider attribute is used to 
indicate to the License Manager that the class is using the 
LicFileLicenseProvider.  The constructor for the control calls 
LicenseManager.Validate to validate that the license is present:</P> 
      
<Code:CodeSnippet runat=server>

<template name="CsTemplate">
    <xmp class=code1>
    
[LicenseProvider(typeof(LicFileLicenseProvider))]
public class MyControl : RichControl {
    private License license = null;

    public MyControl () {
        license = LicenseManager.Validate(typeof(MyControl), this);
    }

    public override void Dispose() {
        if (license != null) {
            license.Dispose();
            license = null;
        }
    }

    protected override void Finalize() {
        Dispose();
        base.Finalize();
    }
            
    </xmp>
</template>

<template name="VBTemplate">
   <xmp class=code1>
   Visual Basic Code not available in the PDC release. Please refer to the C# code.
   </xmp>
</template>

</Code:CodeSnippet>

<P><B>Note:</B> You must dispose of the license when your control is 
disposed.</P>

<Code:CodeSnippet runat=server>
    
    <template name="CsTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/winforms/Samples/Cs/WritingControls/LicensedControl" 
        ViewSource="/quickstart/winforms/Samples/Cs/WritingControls/LicensedControl/LicensedControl.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="C# LicensedControl"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
    <template name="VBTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/winforms/Samples/VB/WritingControls/LicensedControl" 
        ViewSource="/quickstart/winforms/Samples/VB/WritingControls/LicensedControl/LicensedControl.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="VB LicensedControl"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
</Code:CodeSnippet>


<!-- #include virtual="/quickstart/winforms/include/footer.inc" -->
