<!-- #include virtual="/quickstart/winforms/include/header.inc" -->

<h4>Adding Menus to your Win Forms Applications</h4>

<P>Win Forms supports menus and context menus.  Main menus are displayed 
on a menu bar that is located immediately below the title bar of a form.  
The menu bar contains top-level menu items that are used to group related 
sub-menu items.  For example, clicking a File top-level menu item displays 
menu items that are related to file operations.  Menu items are typically 
used to display commands for your application (such as New and Open), but 
they can also be used to display separator bars and submenu items.  You 
can also use check marks next to menu items to display the state of a 
command or a feature in your application.  In Win Forms Main menus are 
represented by the <B>MainMenu</B> control.</P> 
      
<P>Context menus can be displayed for a specific control or area of your 
form.  They are typically accessed using the right mouse button.  In Win 
Forms context menus are represented by the <B>ContextMenu</B> control.</P> 
      
<P>ContextMenu and MainMenu derive from a common Menu class so they both 
share many properties, methods, and events.</P> 
      
<H5>Adding a MainMenu to a Form</H5> 
      
<P>The following code demonstrates how to add a MainMenu to a form:</P> 

<Code:CodeSnippet runat=server>
<template name="CsTemplate">
    <xmp class=code1>
    
    MainMenu mainMenu = new MainMenu();
    this.Menu = mainMenu;
    </xmp>
</template>

<template name="VBTemplate">
   <xmp class=code1>
    
    Dim mainMenu As New MainMenu
    Me.Menu = mainMenu
   </xmp>
</template>

</Code:CodeSnippet>

<h5>Adding a Context Menu to a Control</h5>

<P>The following code demonstrates how to create a ContextMenu and assign 
it to a control:</P> 
      
<Code:CodeSnippet runat=server>
<template name="CsTemplate">
    <xmp class=code1>
    
    ContextMenu label1ContextMenu = new ContextMenu();
    Label label1 = new Label();
    label1.ContextMenu = label1ContextMenu;
    </xmp>
</template>

<template name="VBTemplate">
   <xmp class=code1>
   
    Dim label1ContextMenu As New ContextMenu
    Dim label1 As New Label
    label1.ContextMenu = label1ContextMenu
   </xmp>
</template>

</Code:CodeSnippet>

<h5>Adding a Menu items</h5>
 
<P>The following example adds a File menu item to the MainMenu as well as 
Open and Exit menu items under the File menu item:</P> 

<Code:CodeSnippet runat=server>
<template name="CsTemplate">
    <xmp class=code1>
    
    //Add File Menu
    MenuItem miFile = mainMenu.MenuItems.Add("&File");
    miFile.MenuItems.Add(new MenuItem("&Open..."));
    miFile.MenuItems.Add("-");     // Gives us a seperator
    miFile.MenuItems.Add(new MenuItem("E&xit"));
    </xmp>
</template>

<template name="VBTemplate">
   <xmp class=code1>
   
    'Add File Menu
    Dim miFile As MenuItem = mainMenu.MenuItems.Add("&File")
    miFile.MenuItems.Add(new MenuItem("&Open..."))
    miFile.MenuItems.Add("-")     ' Gives us a seperator
    miFile.MenuItems.Add(new MenuItem("E&xit")) 
   
   </xmp>
</template>
</Code:CodeSnippet>
<br>

<P>The following code demonstrates how to handle the Click event for both 
the Open and Exit menu items created in the previous code example:</P> 

<Code:CodeSnippet runat=server>
<template name="CsTemplate">
    <xmp class=code1>
    
    ....
    'Add File Menu
    Dim miFile As MenuItem = mainMenu.MenuItems.Add("&File")
    miFile.MenuItems.Add(new MenuItem("&Open...", new EventHandler(AddressOf Me.FileOpen_Clicked)))
    miFile.MenuItems.Add("-")     ' Gives us a seperator
    miFile.MenuItems.Add(new MenuItem("E&xit", new EventHandler(AddressOf Me.FileExit_Clicked)))
    ....
    
    'File->Exit Menu item handler
    Private Sub FileExit_Clicked(sender As object, e As System.EventArgs)
        Me.Close
    End Sub

    'File->Open Menu item handler
    Private Sub FileOpen_Clicked(sender As object, e As System.EventArgs)
        MessageBox.Show("And why would this open a file?")
    End Sub

    ....
    </xmp>
</template>

<template name="VBTemplate">
   <xmp class=code1>
   TBD
   </xmp>
</template>
</Code:CodeSnippet>
<br>

<P>The following example demonstrates how to define shortcut keys for the 
menu items created in the previous example:</P> 

<Code:CodeSnippet runat=server>
<template name="CsTemplate">
    <xmp class=code1>
    
    ....
    //Add File Menu
    MenuItem miFile = mainMenu.MenuItems.Add("&File");
    miFile.MenuItems.Add(new MenuItem("&Open...", new EventHandler(this.FileOpen_Clicked), Shortcut.CtrlO));
    miFile.MenuItems.Add("-");     // Gives us a seperator
    miFile.MenuItems.Add(new MenuItem("E&xit", new EventHandler(this.FileExit_Clicked), Shortcut.CtrlX));
    
    ....
    </xmp>
</template>

<template name="VBTemplate">
   <xmp class=code1>
   
    'Add File Menu
    Dim miFile As MenuItem = mainMenu.MenuItems.Add("&File")
    miFile.MenuItems.Add(new MenuItem("&Open...", new EventHandler(AddressOf Me.FileOpen_Clicked), Shortcut.CtrlO))
    miFile.MenuItems.Add("-")     ' Gives us a seperator
    miFile.MenuItems.Add(new MenuItem("E&xit", new EventHandler(AddressOf Me.FileExit_Clicked), Shortcut.CtrlX))

   </xmp>
</template>

</Code:CodeSnippet>

<h5>Adding Sub-Menus</h5>
<P>The following example demonstrates how to create sub-menus:</P>
<Code:CodeSnippet runat=server>
<template name="CsTemplate">
    <xmp class=code1>
    
    //Add Format Menu
    MenuItem miFormat = mainMenu.MenuItems.Add("F&ormat");

    //Font Face sub-menu
    MenuItem miArial = new MenuItem("&Arial", new EventHandler(this.FormatFont_Clicked));
    MenuItem miTimesNewRoman = new MenuItem("&Times New Roman", new EventHandler(this.FormatFont_Clicked));
    MenuItem miTahoma = new MenuItem("&Courier New", new EventHandler(this.FormatFont_Clicked));

    miFormat.MenuItems.Add( "Font &Face"
                          , new EventHandler(this.FormatFont_Clicked)  
                          , (new MenuItem[]{ miArial, miTimesNewRoman, miTahoma })
                          );

    //Font Size sub-menu
    MenuItem miSmall = new MenuItem("&Small", new EventHandler(this.FormatSize_Clicked));
    MenuItem miMedium = new MenuItem("&Medium", new EventHandler(this.FormatSize_Clicked));
    MenuItem miLarge = new MenuItem("&Large", new EventHandler(this.FormatSize_Clicked));

    miFormat.MenuItems.Add( "Font &Size"
                          , new EventHandler(this.FormatSize_Clicked) 
                          , (new MenuItem[]{ miSmall, miMedium, miLarge })
                          );
    </xmp>
</template>

<template name="VBTemplate">
   <xmp class=code1>
   
     'Add Format Menu
     Dim miFormat As MenuItem = mainMenu.MenuItems.Add("F&ormat")

     'Font Face sub-menu
     mmiArial = new MenuItem("&Arial", AddressOf Me.FormatFont_Clicked)
     mmiTimesNewRoman = new MenuItem("&Times New Roman", AddressOf Me.FormatFont_Clicked)
     mmiCourier = new MenuItem("&Courier New", AddressOf Me.FormatFont_Clicked)

     miFormat.MenuItems.Add( "Font &Face"  _
                           , new EventHandler(AddressOf Me.FormatFont_Clicked) _
                           , (new MenuItem(){ mmiArial, mmiTimesNewRoman, mmiCourier }))

     'Font Size sub-menu
     mmiSmall = new MenuItem("&Small", AddressOf Me.FormatSize_Clicked)
     mmiMedium = new MenuItem("&Medium", AddressOf Me.FormatSize_Clicked)
     mmiLarge = new MenuItem("&Large", AddressOf Me.FormatSize_Clicked)

     miFormat.MenuItems.Add( "Font &Size" _
                           , new EventHandler(AddressOf Me.FormatSize_Clicked) _
                           , (new MenuItem(){ mmiSmall, mmiMedium, mmiLarge }))


   </xmp>
</template>
</Code:CodeSnippet>
<br>

<h5>Adding default menu items</h5>
      
<P>The following example demonstrates how to specify a default menu item:</P>
      
<Code:CodeSnippet runat=server>
<template name="CsTemplate">
    <xmp class=code1>
    
    //Add Format Menu
    MenuItem miMedium = new MenuItem("&Medium", new EventHandler(this.FormatSize_Clicked));
    miMedium.DefaultItem = true ;

    </xmp>
</template>

<template name="VBTemplate">
   <xmp class=code1>
   
    'Font Face sub-menu
    mmiArial = new MenuItem("&Arial", AddressOf Me.FormatFont_Clicked)
    mmiArial.DefaultItem = true 

   </xmp>
</template>
</Code:CodeSnippet>
<br>
<h5>Adding check marks to menu items</h5>

<P>The following example demonstrates how to display a check mark next to 
a menu item.  The code also demonstrates how to track which item is 
checked:</P> 

<Code:CodeSnippet runat=server>
<template name="CsTemplate">
    <xmp class=code1>
    
    //Add Format Menu
    MenuItem miMedium = new MenuItem("&Medium", new EventHandler(this.FormatSize_Clicked));
    miMedium.Checked = true ;
    
    ....
    
    private void FormatSize_Clicked(object sender, System.EventArgs e) {
        
        //Track which item is checked
        MenuItem miClicked = (MenuItem)sender;
        miClicked.Checked = true;
        miFormatSizeChecked.Checked = false;
        miFormatSizeChecked = miClicked ;

        //Do Menu item action
        ....
    }


    </xmp>
</template>

<template name="VBTemplate">
   <xmp class=code1>

    'Font Face sub-menu
    mmiArial = new MenuItem("&Arial", AddressOf Me.FormatFont_Clicked)
    mmiArial.Checked = true 
    
    ....
    
    'Format->Font Menu item handler
    Private Sub FormatFont_Clicked(sender As object, e As System.EventArgs)
    
        Dim miClicked As MenuItem = CType(sender, MenuItem)

        miMainFormatFontChecked.Checked = false
        ....
        label1.Font = new Font(fontFace, fontSize)
        
    End Sub
   
   </xmp>
</template>
</Code:CodeSnippet>
<br>

<h5>Cloning menus</h5>

<P>In many cases the context menu for a control is a sub-set of the the 
main menu.  You cannot add the same menu items to multiple menus, but you 
can clone a menu item or set of menu items.  The following code 
demonstrates how to clone the Format menu created previously and adds it 
to the context menu of a label:</P> 
      
<Code:CodeSnippet runat=server>
<template name="CsTemplate">
    <xmp class=code1>
    
    //Add Format to label context menu
    //Note have to add a clone because menus can't belong to 2 parents

    label1ContextMenu.MenuItems.Add(miFormat.CloneMenu());

    </xmp>
</template>

<template name="VBTemplate">
   <xmp class=code1>
   
    'Add Format to label context menu
    'Note have to add a clone because menus can't belong to 2 parents
    label1ContextMenu.MenuItems.Add(miFormat.CloneMenu)

   </xmp>
</template>
</Code:CodeSnippet>
<br>

<h5>Menu Merging</h5>
      
<P>In an MDI application, the main menu of each MDI child form is merged 
into the main menu of the MDI parent form.  See the <a 
href=WinFormsMDI.aspx>MDI QuickStart</a> for an example of menu merging.  
<BR> 

<br>

<h5>Menu Sample</h5>
<P>The following sample illustrates the features mentioned above.
<br><br>
<Code:CodeSnippet runat=server>
    
    <template name="CsTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/winforms/Samples/Cs/Menus" 
        ViewSource="/quickstart/winforms/Samples/Cs/Menus/Menus.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="C# Menus and Context Menus"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
    <template name="VBTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/winforms/Samples/VB/Menus" 
        ViewSource="/quickstart/winforms/Samples/VB/Menus/Menus.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="VB Menus and Context Menus"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
</Code:CodeSnippet>


<!-- #include virtual="/quickstart/winforms/include/footer.inc" -->
