<!-- #include virtual="/quickstart/winforms/include/header.inc" -->


<h4>Printing in Win Forms</h4>

<h5>Important Printing classes:</h5>

<table class=mydd border=0 cellpadding=5>
 <tr>
  <td valign=top><strong>PrintDocument</strong></td>
  <td>
        <p>PrintDocument is used to send output to a printer. You instantiate a PrintDocument, set some
        properties describing what to print, then call the Print method. The PrintDocument raises a 
        PrintPage event for each page to be printed. You add your printing logic to an event handler for
        this event.</p>
  </td>
 </tr>
 <tr>
  <td valign=top><strong>PrinterSettings</strong></td>
  <td>
        <p>Information about how a document should be printed, including which printer to print it on.</p>
  </td>
 </tr>
 <tr>
  <td valign=top><strong>PageSettings</strong></td>
  <td valign=top>
        <p>Information about how a page should be printed.</p>
  </td>
 </tr>
 <tr>
  <td valign=top><strong>PrintPageEventArgs</strong></td>
  <td valign=top>
  <p>The data for the PrintPage event on PrintDocument. It
  provides a clipping rectangle and a Graphics object for the printing surface.</p>
  </td>
 </tr>
 <tr>
  <td valign=top>
  <strong>PrintEventArgs</strong></td>
  <td valign=top>
  <p>The data for the BeginPrint and EndPrint events on
  PrintDocument. Allows the print job to be cancelled.</p>
  </td>
 </tr>
 <tr>
  <td valign=top><strong>PrintDialog</strong></td>
  <td valign=top>
  <p>Printer selection dialog. Wraps the Win32 <i>PrintDlg</i> API</p>
  </td>
 </tr>
 <tr>
  <td valign=top><strong>PageSetupDialog</strong></td>
  <td valign=top>
  <p>Page properties dialog. Wraps the Win32 <i>PageSetupDlg</i> API</p>
  </td>
 </tr>
 <tr>
  <td valign=top><strong>PrintPreviewControl</strong></td>
  <td valign=top>
  <p>A control that displays a PrintDocument. Allows the
  creation of a print preview dialog.</p>
  </td>
 </tr>
 <tr>
  <td valign=top><strong>PrintPreviewDialog</strong></td>
  <td valign=top>
  <p>A dialog that displays a PrintDocument using the <i>PrintPreviewControl</i></p>
  </td>
 </tr>
 <tr>
  <td valign=top><strong>PrintController</strong></td>
  <td valign=top>
  <p>A PrintController is the &quot;engine&quot; that controls how a PrintDocument is printed. <i>PrintDocument.Print</i>
  uses a print control to render the document.</p>
  
  <p>The framework provides 2 print controllers:</p>
  <ul>
  <li><i>DefaultPrintController</i> renders to a printer</li>
  <li><i>PreviewPrintController</i> renders to the print preview control</li>
  </ul>
  <p>Typically you will never have implement a PrintController. You would only
  implement a PrintController if you want to render to a different surface.</p>
  </td>
 </tr>
</table>

<h5>Building an application that prints</h5>
<p>The easiest way to understand how the printing classes fit
together is to work through a simple example: printing the contents of a text
file to a printer</p>

<h6>Step 1: Print Logic</h6>
<p>The first thing you need to do is to write your printing
logic. You do this by handling events on a PrintDocument. When the PrintDocument.Print() method is called 
the following events are fired:

<ul>
<li>BeginPrint</li>
<li>PagePrint (one or more)</li>
<li>EndPrint</li>
</ul>

<p>The PagePrint event arguments type (PagePrintEventArgs) has a HasMorePages property. If this is set to true when your
event handler returns, PrintDocument set up a new page and raises the PagePrint event again. 
<p>Therefore the logic in your PagePrint event handler is basically the following:
<ul>
<li>Print the contents of the page using the information in the event arguments. The event 
arguments contain the Graphics for the printer, the PageSettings for that page,
the bounds of the page and the bounds of the margins.
<li>Determine whether there are more pages to print                            
<li>If there are set HasMorePages to true 
<li>If there aren't set HasMorePages to false
</ul>
<p>In the simplest case you can create a PrintDocument and handle the PagePrint event as part
of the Form that contains the print request:</p>

<Code:CodeSnippet runat=server>

<template name="CsTemplate">
    <xmp class=code1>

      //Example Form that Prints</span>
        public class PrintCtl : Form {

            Private string[] printBuffer ;
            Private Font printFont;
            Private int startLine;

            //Event fired when the user presses the print button</span>
            private void print_Click(object sender, EventArgs e) {
                
                fileToPrint = new StreamReader ("PrintMe.Txt");
                try {
                    printFont = new Font("Arial", 10);
                    PrintDocument pd = new PrintDocument(); //Assumes the default printer</span>
                    pd.AddOnPrintPage(new PrintPageEventHandler(this.pd_PrintPage));
                    pd.Print();
                } finally {
                    fileToPrint.Close() ;
                }
                
            }

            //Event fired for each page to print</span>
            private void pd_PrintPage(object sender, PrintPageEventArgs ev) {
               float lpp = 0 ;
               float yPos =  0 ;
               int count = 0 ;
               float leftMargin = ev.MarginBounds.Left;
               float topMargin = ev.MarginBounds.Top;
               String line=null;
               
               //Work out the number of lines per page </span>
               //Use the MarginBounds on the event to do this</span>
               lpp = ev.MarginBounds.Height  / printFont.GetHeight(ev.Graphics) ;</span>

               //Now iterate over the file printing out each line </span>
               //NOTE WELL: This assumes that a single line is not wider than the page width </span>
               //Check count first so that we don't read line that we won't print </span>
               while (count < lpp && ((line=streamToPrint.ReadLine()) != null)) {
                   yPos = topMargin + (count * printFont.GetHeight(ev.Graphics));
                
                   //NOTE WELL: In the PDC Release you must pass a StringFormat to DrawString or the 
                   //Print Preview control will not work. 
                   ev.Graphics.DrawString (line, printFont, Brushes.Black, leftMargin, yPos, new StringFormat());
                   
                   count++;
               }

               //If we have more lines then print another page </span>
               if (line != null) 
                   ev.HasMorePages = true ;
               else 
                   ev.HasMorePages = false ;
            }
            
            ....
        }
              
    </xmp>
</template>

<template name="VBTemplate">

   <xmp class=code1>

    Public Class PrintingExample1 
        Inherits System.WinForms.Form 

        ....
                
        private printFont As Font
        private streamToPrint As StreamReader 

        Public Sub New () 
            MyBase.New
            InitializeComponent()
        End Sub

        'Event fired when the user presses the print button
        Private Sub printButton_Click(sender As object, e As System.EventArgs)
        
            Try 
                streamToPrint = new StreamReader ("PrintMe.Txt")
                Try
                    printFont = new Font("Arial", 10)
                    Dim pd as PrintDocument = new PrintDocument() 'Assumes the default printer
                    pd.AddOnPrintPage(AddressOf Me.pd_PrintPage)
                    pd.Print()
                Finally
                    streamToPrint.Close() 
                End Try

            Catch ex As Exception
                MessageBox.Show("An error occurred printing the file - " + ex.Message)
            End Try
            
        End Sub

        'Event fired for each page to print
        Private Sub pd_PrintPage(sender As object, ev As System.Drawing.Printing.PrintPageEventArgs)
        
            Dim lpp As Single = 0 
            Dim yPos As Single =  0 
            Dim count As Integer = 0 
            Dim leftMargin As Single = ev.MarginBounds.Left
            Dim topMargin As Single = ev.MarginBounds.Top
            Dim line as String
            
            'Work out the number of lines per page 
            'Use the MarginBounds on the event to do this
            lpp = ev.MarginBounds.Height  / printFont.GetHeight(ev.Graphics) 

            'Now iterate over the file printing out each line
            'NOTE WELL: This assumes that a single line is not wider than the page width
            'Check count first so that we don't read line that we won't print
            line=streamToPrint.ReadLine()
            while (count < lpp AND line <> Nothing)
                
                yPos = topMargin + (count * printFont.GetHeight(ev.Graphics))
                
                'NOTE WELL: In the PDC Release you must pass a StringFormat to DrawString or the 
                'Print Preview control will not work. 
                ev.Graphics.DrawString (line, printFont, Brushes.Black, leftMargin, yPos, new StringFormat())
                
                count = count + 1

                if (count < lpp) then
                    line=streamToPrint.ReadLine()
                end if
                    
            End While

            'If we have more lines then print another page
            If (line <> Nothing) Then
                ev.HasMorePages = True 
            Else 
                ev.HasMorePages = False 
            End If 
            
        End Sub

        ....

    End Class
           
   </xmp>
</template>
</Code:CodeSnippet>


<p>To view and run this sample:
<br><br>
<Code:CodeSnippet runat=server>
    
    <template name="CsTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/winforms/Samples/Cs/Printing/Example1" 
        ViewSource="/quickstart/winforms/Samples/Cs/Printing/Example1/PrintingExample1.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="C# Simple Printing Example1"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
    <template name="VBTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/winforms/Samples/VB/Printing/Example1" 
        ViewSource="/quickstart/winforms/Samples/VB/Printing/Example1/PrintingExample1.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="VB Simple Printing Example1"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
</Code:CodeSnippet>
<br>

<h6>Step 2: Defining your own Print Document</h6>

<p >For a more complex print job or a print logic that you wish to reuse across multiple forms you
would derive a new class from <i>PrintDocument</i> and encapsulate your printing logic in that.
In this case you handle the PrintPage event by overriding the "OnPrintPage" method rather than using an
event handler.</p>
<br>
<Code:CodeSnippet runat=server>

<template name="CsTemplate">
    <xmp class=code1>
    public class TextFilePrintDocument : PrintDocument {

        private Font printFont           = null ;
        private StreamReader streamToPrint = null ;

        public TextFilePrintDocument(StreamReader streamToPrint) : base ()  {
            this.streamToPrint = streamToPrint ;
        }

        //Override OnBeginPrint to set up the font we are going to use</span>
        protected override void OnBeginPrint(PrintEventArgs ev) {
            base.OnBeginPrint(ev) ;
            printFont = new Font("Arial", 10);
        }

        //Override the OnPrintPage to provide the printing logic for the document</span>
        protected override void OnPrintPage(PrintPageEventArgs ev) {
            
            base.OnPrintPage(ev) ;
            
            float lpp = 0 ;
            float yPos =  0 ;
            int count = 0 ;
            float leftMargin = ev.MarginBounds.Left;
            float topMargin = ev.MarginBounds.Top;
            String line=null;
            
            //Work out the number of lines per page </span>
            //Use the MarginBounds on the event to do this </span>
            lpp = ev.MarginBounds.Height  / printFont.GetHeight(ev.Graphics) ;

            //Now iterate over the file printing out each line</span>
            //NOTE WELL: This assumes that a single line is not wider than the page width</span>
            //Check count first so that we don't read line that we won't print</span>
            while (count < lpp && ((line=streamToPrint.ReadLine()) != null)) {
                yPos = topMargin + (count * printFont.GetHeight(ev.Graphics));

                //NOTE WELL: In the PDC Release you must pass a StringFormat to DrawString or the 
                //Print Preview control will not work. 
                ev.Graphics.DrawString (line, printFont, Brushes.Black, leftMargin, yPos, new StringFormat());

                count++;
            }

            //If we have more lines then print another page</span>
            if (line != null) 
                ev.HasMorePages = true ;
            else 
                ev.HasMorePages = false ;
        }

    }
    </xmp>
</template>

<template name="VBTemplate">

   <xmp class=code1>

Visual Basic in line code sample not available for the PDC Release. Refer to the complete Visual Basic sample.
     
   </xmp>
</template>
</Code:CodeSnippet>


<p>To view and run this sample:
<br><br>
<Code:CodeSnippet runat=server>
    
    <template name="CsTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/winforms/Samples/Cs/Printing/Example2" 
        ViewSource="/quickstart/winforms/Samples/Cs/Printing/Example2/PrintingExample2.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="C# Simple Printing Example2"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
    <template name="VBTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/winforms/Samples/VB/Printing/Example2" 
        ViewSource="/quickstart/winforms/Samples/VB/Printing/Example2/PrintingExample2.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="VB Simple Printing Example2"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
</Code:CodeSnippet>
<br>

<h6>Step 3: Letting the user choose which printer to use</h6>

<p>Now you have your print logic working the next step is to allow the user to choose a printer using the 
Windows Print Dialog. To do this you create a PrintDocument and pass it to the PrintDialog:
</p>
<br>
<Code:CodeSnippet runat=server>

<template name="CsTemplate">
    <xmp class=code1>
    //Event fired when the user presses the print button</span>
    private void printButton_Click(object sender, EventArgs e) {
        StreamReader streamToPrint = new StreamReader ("PrintMe.Txt");
        try {
            TextFilePrintDocument pd = new TextFilePrintDocument(streamToPrint); //Assumes the default printer</span>
            
            PrintDialog dlg = new PrintDialog() ;
            dlg.Document = pd;
            DialogResult result = dlg.ShowDialog();
            
            if (result == DialogResult.OK) {
                pd.Print();
            }

        } finally {
            streamToPrint.Close() ;
        }
    }
    </xmp>
</template>

<template name="VBTemplate">

   <xmp class=code1>

Visual Basic in line code sample not available for the PDC Release. Refer to the complete Visual Basic sample.
     
   </xmp>
</template>
</Code:CodeSnippet>


<p>To view and run this sample:
<br><br>
<Code:CodeSnippet runat=server>
    
    <template name="CsTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/winforms/Samples/Cs/Printing/Example3" 
        ViewSource="/quickstart/winforms/Samples/Cs/Printing/Example3/PrintingExample3.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="C# Simple Printing Example3"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
    <template name="VBTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/winforms/Samples/VB/Printing/Example3" 
        ViewSource="/quickstart/winforms/Samples/VB/Printing/Example3/PrintingExample3.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="VB Simple Printing Example3"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
</Code:CodeSnippet>
<br>

<h6>Step 4: Letting the user choose page settings</h6>

<p>Once the user can choose a printer and print their document you may want to let them choose 
page settings such as the orientation of the paper or the margins to use. To do this you create
a PageSettings instance and pass it to the PageSetupDialog:
</p>
<br>
<Code:CodeSnippet runat=server>

<template name="CsTemplate">
    <xmp class=code1>
    //Event fired when the user presses the page setup button</span>
    private void pageSetupButton_Click(object sender, EventArgs e) {
        
        PageSetupDialog psDlg = new PageSetupDialog() ;
        
        if (storedPageSettings == null) {
            storedPageSettings =  new PageSettings();
        }

        psDlg.PageSettings = storedPageSettings ;
        psDlg.ShowDialog();
    }
    </xmp>
</template>

<template name="VBTemplate">

   <xmp class=code1>

Visual Basic in line code sample not available for the PDC Release. Refer to the complete Visual Basic sample.
     
   </xmp>
</template>
</Code:CodeSnippet>

<p>You can then use this custom PageSettings instance when the user prints the document:</p>
<br>
<Code:CodeSnippet runat=server>

<template name="CsTemplate">
    <xmp class=code1>
    //Event fired when the user presses the print button</span>
    private void printButton_Click(object sender, EventArgs e) {
        StreamReader streamToPrint = new StreamReader ("PrintMe.Txt");
        try {
            TextFilePrintDocument pd = new TextFilePrintDocument(streamToPrint); 

            if (storedPageSettings != null) {
                pd.DefaultPageSettings = storedPageSettings ;
            }
            
            PrintDialog dlg = new PrintDialog() ;
            dlg.Document = pd;
            DialogResult result = dlg.ShowDialog();
            
            if (result == DialogResult.OK) {
                pd.Print();
            }

        } finally {
            streamToPrint.Close() ;
        }
    }
    </xmp>
</template>

<template name="VBTemplate">

   <xmp class=code1>

Visual Basic in line code sample not available for the PDC Release. Refer to the complete Visual Basic sample.
     
   </xmp>
</template>
</Code:CodeSnippet>


<p>To view and run this sample:
<br><br>
<Code:CodeSnippet runat=server>
    
    <template name="CsTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/winforms/Samples/Cs/Printing/Example4" 
        ViewSource="/quickstart/winforms/Samples/Cs/Printing/Example4/PrintingExample4.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="C# Simple Printing Example4"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
    <template name="VBTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/winforms/Samples/VB/Printing/Example4" 
        ViewSource="/quickstart/winforms/Samples/VB/Printing/Example4/PrintingExample4.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="VB Simple Printing Example4"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
</Code:CodeSnippet>
<br>


<h6>Step 5: Displaying a Print Preview Window</h6>

<p>The print preview window allows a user to preview their document before printing it. You can add 
a print preview window to your application by creating a PrintDocument and passing it to the Print
Preview Dialog:
</p>
<br>
<Code:CodeSnippet runat=server>

<template name="CsTemplate">
    <xmp class=code1>
    //Event fired when the user presses the print preview button</span>
    private void printPreviewButton_Click(object sender, EventArgs e) {
            
        StreamReader streamToPrint = new StreamReader ("PrintMe.Txt");
        try {
            TextFilePrintDocument pd = new TextFilePrintDocument(streamToPrint); 

            if (storedPageSettings != null) {
                pd.DefaultPageSettings = storedPageSettings ;
            }
            
            PrintPreviewDialog dlg = new PrintPreviewDialog() ;
            dlg.Document = pd;
            dlg.ShowDialog();

        } finally {
            streamToPrint.Close() ;
        }
    }

    </xmp>
</template>

<template name="VBTemplate">

   <xmp class=code1>

Visual Basic in line code sample not available for the PDC Release. Refer to the complete Visual Basic sample.
     
   </xmp>
</template>
</Code:CodeSnippet>

<p>To view and run this sample:
<br><br>
<Code:CodeSnippet runat=server>
    
    <template name="CsTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/winforms/Samples/Cs/Printing/Example5" 
        ViewSource="/quickstart/winforms/Samples/Cs/Printing/Example5/PrintingExample5.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="C# Simple Printing Example5"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
    <template name="VBTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/winforms/Samples/VB/Printing/Example5" 
        ViewSource="/quickstart/winforms/Samples/VB/Printing/Example5/PrintingExample5.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="VB Simple Printing Example5"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
</Code:CodeSnippet>
<br>

<h5>Putting it all together</h5>

<p>The Win Forms samples include a sample called <I>SimplePad</I> that shows how to build a simple 
text editor that prints the file being edited.</p>
<br>
<Code:CodeSnippet runat=server>
    
    <template name="CsTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/winforms/Samples/Cs/Printing/SimplePad" 
        ViewSource="/quickstart/winforms/Samples/Cs/Printing/SimplePad/SimplePad.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="C# SimplePad"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
    <template name="VBTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/winforms/Samples/VB/Printing/SimplePad" 
        ViewSource="/quickstart/winforms/Samples/VB/Printing/SimplePad/SimplePad.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="VB SimplePad"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
</Code:CodeSnippet>



<!-- #include virtual="/quickstart/winforms/include/footer.inc" -->
