//------------------------------------------------------------------------------
/// <copyright from='1997' to='2001' company='Microsoft Corporation'>           
///    Copyright (c) Microsoft Corporation. All Rights Reserved.                
///       
///    This source code is intended only as a supplement to Microsoft
///    Development Tools and/or on-line documentation.  See these other
///    materials for detailed information regarding Microsoft code samples.
///
/// </copyright>                                                                
//------------------------------------------------------------------------------
namespace ProgressBarCtl {
    using System;
    using System.Collections;
    using System.ComponentModel;
    using System.Drawing;
    using System.WinForms;
    using System.Threading;

    // <doc>
    // <desc>
    //     This class demonstrates the ProgressBar control.
    //     The ProgressBar is updated periodically via another thread based on the 
    //     settings in this control
    // </desc>
    // </doc>
    //
    public class ProgressBarCtl : System.WinForms.Form {
        private int     iSleepTime ;
        private Thread  timedProgress ;

        public ProgressBarCtl() : base() {
             	
            // This call is required for support of the WFC Form Designer.
            InitializeComponent();

            iSleepTime = 100 ;
            cmbMinimum.SelectedIndex = 0 ;
            progbar.Minimum = 0 ;
            cmbMaximum.SelectedIndex = 0 ;
            progbar.Maximum = 50 ;
            cmbStep.SelectedIndex = 0 ;
            progbar.Step = 1 ;
    
            // Spin off a new thread to update the ProgressBar control
            timedProgress = new Thread(new ThreadStart(TimedProgressProc));
            timedProgress.IsBackground = true;
            timedProgress.Start();

        }
    
        // <doc>
        // <desc>
        //     This code executes on the WinForms thread.
        // </desc>
        // </doc>
        //
        private void UpdateProgress() {
            int min ;
            double numerator, denominator, completed ;
    
            //Reset to start if required
            if (progbar.Value == progbar.Maximum) {
                progbar.Value = progbar.Minimum ;
            }

            progbar.PerformStep() ;
            lblValue.Text = progbar.Value.ToString();
    
            min         = progbar.Minimum ;
            numerator   = progbar.Value - min ;
            denominator = progbar.Maximum - min ;
            completed   = (numerator / denominator) * 100.0 ;
    
            lblCompleted.Text = Math.Round(completed).ToString() + "%" ;
        }
    
        // <doc>
        // <desc>
        //     This function runs in the timedProgress thread and updates the
        //     ProgressBar on the form.
        // </desc>
        // </doc>
        //
        private void TimedProgressProc() {
            try {
                MethodInvoker mi = new MethodInvoker(UpdateProgress);
                while (true) {
                    Invoke(mi);
                    int iSleepTime = this.SleepTime;
                    Thread.Sleep(iSleepTime) ;
                }
            }
            //Thrown when the thread is interupted by the main thread - exiting the loop
            catch (ThreadInterruptedException e) {
                if (e != null) {}
            }
            catch (Exception we) {
                if (we != null) {}
            }
        }
    
        // <doc>
        // <desc>
        //     Property controlling the progress of the progress bar - used by the background thread
        // </desc>
        // </doc>
        //
        private int SleepTime {
            get {
                lock(this) {
                    return iSleepTime ;
                }
            }
            set {
                lock(this) {
                    iSleepTime = value ;
                }
            }
        }
    
        // <doc>
        // <desc>
        //     ProgressBarCtl overrides dispose so it can clean up the
        //     component list.
        // </desc>
        // </doc>
        //
        public override void Dispose() {
            /*
             * We have to make sure that our thread doesn't attempt
             * to access our controls after we dispose them.
             */
            if (timedProgress != null) {
                timedProgress.Interrupt();
                timedProgress = null;
            }
    
            base.Dispose();
            components.Dispose();
        }
    
        private void sldrSpeed_Scroll(object sender, EventArgs e) {
            TrackBar tb = (TrackBar) sender ;
            int time = 110 - tb.Value ;
            this.SleepTime = time ;
        }
    
        private void cmbMinimum_SelectedIndexChanged(object sender, EventArgs e) {
            try {
                int newMinValue = Int32.Parse((string)cmbMinimum.SelectedItem);
                if (newMinValue < Int32.Parse((string)cmbMaximum.SelectedItem))
                    progbar.Minimum = newMinValue;
            }
            catch (Exception ex) {
                // thrown if Int32.Parse can't convert
                if (ex !=null) {}
            }
        }
    
        private void cmbMaximum_SelectedIndexChanged(object sender, EventArgs e) {
            try {
                int newMaxValue = Int32.Parse((string)cmbMaximum.SelectedItem);
                if (newMaxValue > Int32.Parse((string)cmbMinimum.SelectedItem))
                    progbar.Maximum = newMaxValue;
            }
            catch (Exception ex) {
                // thrown if Int32.Parse can't convert
                if (ex !=null) {}
            }
        }
    
        private void cmbStep_SelectedIndexChanged(object sender, EventArgs e) {
            try {
                progbar.Step = Int32.Parse((string)cmbStep.SelectedItem);
            }
            catch (Exception ex) {
                // thrown if Int32.Parse can't convert
                if (ex !=null) {}
            }
        }
    
        // NOTE: The following code is required by the WFC Form Designer
        // It can be modified using the WFC Form Designer.  
        // Do not modify it using the code editor.
        private System.ComponentModel.Container components;
        private System.WinForms.Label label3; 
        private System.WinForms.Label lblCompleted; 
        private System.WinForms.TrackBar sldrSpeed; 
        private System.WinForms.ProgressBar progbar; 
        private System.WinForms.Label label1; 
        private System.WinForms.ComboBox cmbMinimum; 
        private System.WinForms.Label label2; 
        private System.WinForms.Label label5; 
        private System.WinForms.ComboBox cmbMaximum; 
        private System.WinForms.GroupBox grpBehavior; 
        private System.WinForms.Label label4; 
        private System.WinForms.Label label6; 
        private System.WinForms.Label lblValue; 
        private System.WinForms.ComboBox cmbStep; 
    
        private void InitializeComponent() {
            this.components = new System.ComponentModel.Container();
            this.label3 = new System.WinForms.Label();
            this.lblCompleted = new System.WinForms.Label();
            this.sldrSpeed = new System.WinForms.TrackBar();
            this.progbar = new System.WinForms.ProgressBar();
            this.label1 = new System.WinForms.Label();
            this.cmbMinimum = new System.WinForms.ComboBox();
            this.label2 = new System.WinForms.Label();
            this.label5 = new System.WinForms.Label();
            this.cmbMaximum = new System.WinForms.ComboBox();
            this.grpBehavior = new System.WinForms.GroupBox();
            this.label4 = new System.WinForms.Label();
            this.label6 = new System.WinForms.Label();
            this.lblValue = new System.WinForms.Label();
            this.cmbStep = new System.WinForms.ComboBox();

            this.Size = new System.Drawing.Size(512, 300);
            this.Text = "ProgressBar";
    
            label3.Location = new System.Drawing.Point(16, 72);
            label3.Size = new System.Drawing.Size(48, 16);
            label3.TabIndex = 6;
            label3.TabStop = false;
            label3.Text = "step:";
    
            lblCompleted.Location = new System.Drawing.Point(128, 56);
            lblCompleted.Size = new System.Drawing.Size(56, 16);
            lblCompleted.TabIndex = 2;
            lblCompleted.TabStop = false;
            lblCompleted.Text = "";
    
            sldrSpeed.Location = new System.Drawing.Point(16, 216);
            sldrSpeed.Size = new System.Drawing.Size(216, 42);
            sldrSpeed.TabIndex = 1;
            sldrSpeed.TabStop = false;
            sldrSpeed.Text = "trackBar1";
            sldrSpeed.Minimum = 10;
            sldrSpeed.Maximum = 100;
            sldrSpeed.Value = 10;
            sldrSpeed.TickFrequency = 10;
            sldrSpeed.AddOnScroll(new EventHandler(this.sldrSpeed_Scroll));
            sldrSpeed.BackColor = System.Drawing.SystemColors.Control ; //Fix for bad painting
    
            progbar.BackColor = System.Drawing.SystemColors.Control;
            progbar.Location = new System.Drawing.Point(24, 24);
            progbar.Size = new System.Drawing.Size(192, 16);
            progbar.TabIndex = 0;
            progbar.Text = "progbar";
            progbar.Step = 1;
    
            label1.Location = new System.Drawing.Point(16, 48);
            label1.Size = new System.Drawing.Size(56, 16);
            label1.TabIndex = 4;
            label1.TabStop = false;
            label1.Text = "Maximum:";
    
            cmbMinimum.Location = new System.Drawing.Point(136, 24);
            cmbMinimum.Size = new System.Drawing.Size(96, 21);
            cmbMinimum.TabIndex = 3;
            cmbMinimum.Text = "";
            cmbMinimum.Style = System.WinForms.ComboBoxStyle.DropDownList;
            cmbMinimum.Items.All = (object[])new object[] {"0", "10", "40"};
            cmbMinimum.AddOnSelectedIndexChanged(new EventHandler(this.cmbMinimum_SelectedIndexChanged));
    
            label2.Location = new System.Drawing.Point(16, 24);
            label2.Size = new System.Drawing.Size(48, 16);
            label2.TabIndex = 2;
            label2.TabStop = false;
            label2.Text = "Minimum:";
    
            label5.Location = new System.Drawing.Point(24, 56);
            label5.Size = new System.Drawing.Size(100, 16);
            label5.TabIndex = 1;
            label5.TabStop = false;
            label5.Text = "Percent Completed:";
    
            cmbMaximum.Location = new System.Drawing.Point(136, 48);
            cmbMaximum.Size = new System.Drawing.Size(96, 21);
            cmbMaximum.TabIndex = 5;
            cmbMaximum.Text = "";
            cmbMaximum.Style = System.WinForms.ComboBoxStyle.DropDownList;
            cmbMaximum.Items.All = (object[])new object[] {"50", "100", "200"};
            cmbMaximum.AddOnSelectedIndexChanged(new EventHandler(this.cmbMaximum_SelectedIndexChanged));
    
            grpBehavior.Location = new System.Drawing.Point(248, 16);
            grpBehavior.Size = new System.Drawing.Size(248, 264);
            grpBehavior.TabIndex = 5;
            grpBehavior.TabStop = false;
            grpBehavior.Text = "ProgressBar";
    
            label4.Location = new System.Drawing.Point(16, 192);
            label4.Size = new System.Drawing.Size(100, 16);
            label4.TabIndex = 0;
            label4.TabStop = false;
            label4.Text = "Completion Speed:";
    
            label6.Location = new System.Drawing.Point(24, 80);
            label6.Size = new System.Drawing.Size(100, 16);
            label6.TabIndex = 3;
            label6.TabStop = false;
            label6.Text = "Value:";
    
            lblValue.Location = new System.Drawing.Point(128, 80);
            lblValue.Size = new System.Drawing.Size(56, 16);
            lblValue.TabIndex = 4;
            lblValue.TabStop = false;
            lblValue.Text = "";
    
            cmbStep.Location = new System.Drawing.Point(136, 72);
            cmbStep.Size = new System.Drawing.Size(96, 21);
            cmbStep.TabIndex = 7;
            cmbStep.Text = "";
            cmbStep.Style = System.WinForms.ComboBoxStyle.DropDownList;
            cmbStep.Items.All = (object[])new object[] {"1", "5", "10", "20"};
            cmbStep.AddOnSelectedIndexChanged(new EventHandler(this.cmbStep_SelectedIndexChanged));
    
            this.Controls.Add(grpBehavior);
            this.Controls.Add(lblValue);
            this.Controls.Add(label6);
            this.Controls.Add(lblCompleted);
            this.Controls.Add(label5);
            this.Controls.Add(progbar);
            grpBehavior.Controls.Add(cmbStep);
            grpBehavior.Controls.Add(cmbMaximum);
            grpBehavior.Controls.Add(cmbMinimum);
            grpBehavior.Controls.Add(label3);
            grpBehavior.Controls.Add(label1);
            grpBehavior.Controls.Add(label2);
            grpBehavior.Controls.Add(sldrSpeed);
            grpBehavior.Controls.Add(label4);
        }
    

        // The main entry point for the application.
        public static void Main(string[] args) {
            Application.Run(new ProgressBarCtl());
        }

    }
}

